import os
import importlib.util
from termcolor import cprint as print

from ..enums import EnvironmentType, HookTypes
from ..ihook import IHook


class HooksManager(object):
	""" Manager for the executing of all hooks
	"""
	def __init__(self):
		""" Initialize
		"""
		# A place to save all the hook per type
		self.hooks = {}

	def readHooks(self):
		""" Read all the hooks
		"""
		# Check if there is an hooks directory
		if os.path.exists(".pete/hooks") is False:
			return

		# Read the hook files
		hooks = os.listdir(".pete/hooks")

		print("Found %s hooks, checking now" % len(hooks), "blue")

		# Walk throught the hooks
		for fileName in hooks:
			# Check if it is an py file
			if os.path.splitext(fileName)[1] != ".py":
				continue

			# Import the file
			spec = importlib.util.spec_from_file_location(os.path.splitext(fileName)[0], os.path.join(".pete", "hooks", fileName))
			module = importlib.util.module_from_spec(spec)
			spec.loader.exec_module(module)

			# Create a hook
			try:
				hook: IHook = module.Hook()
			except AttributeError:
				print("There is no Hook class in %s" % fileName, "red")
				return False

			# Check if it is an DeploymentHook
			if isinstance(hook, IHook) is False:
				print("Hook %s is not a DeploymentHook subclass" % fileName, "red")
				return False

			# Check if there is an Environments is a list
			if isinstance(hook.ON_ENVIRONMENT, list) is False:
				print("Hook %s does not have a list of environments to be executed on" % fileName, "red")
				return False

			# Loop trough the types
			for environmentType in hook.ON_ENVIRONMENT:
				# Check if it is an HookType
				if isinstance(environmentType, EnvironmentType) is False:
					print("Hook %s does not have a valid EnvironmentYype" % fileName, "red")
					return False

				# Check if the type exists
				if environmentType not in self.hooks:
					# Add the type to the hooks
					self.hooks[environmentType] = {}

				# Check if the HookType is the valid type
				if isinstance(hook.ON_TYPE, HookTypes) is False:
					print("Hook %s does not have a valid HookType" % fileName, "red")
					return False

				# Check if the hook types exists in the hooks
				if hook.ON_TYPE not in self.hooks[environmentType]:
					# Add the hook type
					self.hooks[environmentType][hook.ON_TYPE] = []

				# Add the hook
				self.hooks[environmentType][hook.ON_TYPE].append(hook)

	def executeHooks(self, environmentType: EnvironmentType, hookType: HookTypes):
		""" Execute the hooks
		"""
		# Check if the environment type exists
		if environmentType not in self.hooks:
			# Stop right here, there is nothing to do
			return

		# Check if the hook type exists
		if hookType not in self.hooks[environmentType]:
			# Stop right here, there is nothing to do
			return

		# Loop trough the hooks
		for hook in self.hooks[environmentType][hookType]:
			# Tell the people we are running this hook
			print("Executing hook %s" % hook.NAME, "blue")

			# Execute the hook
			hook.execute(environmentType)


# Make the HooksManager a singleton
hooksManager = HooksManager()



import sys

from PyInquirer import prompt, Separator
from termcolor import cprint as print

from .iaction import IAction
from ..tools.configuration import ConfigurationTool, ConfigKey, ConfigType
from ..validator import Validator


class ConfigureAction(IAction):
	""" The configure action
	"""

	def start(self):
		""" Start the configure action
		"""
		# Just a friendly message
		print("Lets configure TPD Pete, I will now ask you some questions. Feel free to answer them.\n", "yellow")

		# Check if there is already a config
		if Validator.hasPeteSetup() is True:
			ConfigurationTool.readConfig(getGlobal=True, getProject=False, getLocal=False)

		# Fill temporary config
		self.config = {}
		self.config[ConfigKey.DEV_PROFILE] = self._getConfigValue(ConfigKey.DEV_PROFILE)
		self.config[ConfigKey.PROD_PROFILE] = self._getConfigValue(ConfigKey.PROD_PROFILE)
		self.config[ConfigKey.DEV_BUCKET] = self._getConfigValue(ConfigKey.DEV_BUCKET)
		self.config[ConfigKey.PROD_BUCKET] = self._getConfigValue(ConfigKey.PROD_BUCKET)
		self.config[ConfigKey.DEV_REGION] = self._getConfigValue(ConfigKey.DEV_REGION)
		self.config[ConfigKey.PROD_REGION] = self._getConfigValue(ConfigKey.PROD_REGION)

		# Keep showing main config
		while True:
			answer = self._showMainConfig()

			if answer == 1:
				# Ask development profile
				defaultValue = self._getConfigValue(ConfigKey.DEV_PROFILE)
				value = self._askAWSProfile(default=defaultValue)
				self.setValue(ConfigKey.DEV_PROFILE, value)

			elif answer == 2:
				# Ask development bucket
				profile = self._getConfigValue(ConfigKey.DEV_PROFILE)
				if profile is None:
					print("Configure the profile first", "orange")
					continue
				defaultValue = self._getConfigValue(ConfigKey.DEV_BUCKET)
				value = self._askS3Bucket(profile=profile, default=defaultValue)
				self.setValue(ConfigKey.DEV_BUCKET, value)

			elif answer == 3:
				# Ask development region
				profile = self._getConfigValue(ConfigKey.DEV_PROFILE)
				if profile is None:
					print("Configure the profile first", "orange")
					continue
				defaultValue = self._getConfigValue(ConfigKey.DEV_REGION)
				value = self._askAWSRegion(profile=profile, default=defaultValue)
				self.setValue(ConfigKey.DEV_REGION, value)

			elif answer == 5:
				# Ask production profile
				defaultValue = self._getConfigValue(ConfigKey.PROD_PROFILE)
				value = self._askAWSProfile(default=defaultValue)
				self.setValue(ConfigKey.PROD_PROFILE, value)

			elif answer == 6:
				# Ask production bucket
				profile = self._getConfigValue(ConfigKey.PROD_PROFILE)
				if profile is None:
					print("Configure the profile first", "orange")
					continue
				defaultValue = self._getConfigValue(ConfigKey.PROD_BUCKET)
				value = self._askS3Bucket(profile=profile, default=defaultValue)
				self.setValue(ConfigKey.PROD_BUCKET, value)

			elif answer == 7:
				# Ask production region
				profile = self._getConfigValue(ConfigKey.PROD_PROFILE)
				if profile is None:
					print("Configure the profile first", "orange")
					continue
				defaultValue = self._getConfigValue(ConfigKey.PROD_REGION)
				value = self._askAWSRegion(profile=profile, default=defaultValue)
				self.setValue(ConfigKey.PROD_REGION, value)

			elif answer == 8:
				# Save the config
				saved = self._saveConfig()
				if saved is True:
					sys.exit()

	def _showMainConfig(self):
		""" Show the main config options
		"""
		# Make the options for the list
		choices = []
		choices.append(Separator("~ Development ~"))
		choices.append("  AWS profile: %s" % self.config[ConfigKey.DEV_PROFILE])
		choices.append("  AWS bucket: %s" % self.config[ConfigKey.DEV_BUCKET])
		choices.append("  AWS region: %s" % self.config[ConfigKey.DEV_REGION])
		choices.append(Separator("~ Production ~"))
		choices.append("  AWS profile: %s" % self.config[ConfigKey.PROD_PROFILE])
		choices.append("  AWS bucket: %s" % self.config[ConfigKey.PROD_BUCKET])
		choices.append("  AWS region: %s" % self.config[ConfigKey.PROD_REGION])
		choices.append("Done")

		# Show the list
		answer = prompt({
			"type": "list",
			"name": "option",
			"message": "Choose an option to configure",
			"choices": choices
		})

		# Check if answer is empty
		if answer == {}:
			sys.exit()

		# Get the index of the option
		index = choices.index(answer['option'])

		return index

	def _getConfigValue(self, key):
		""" Get the config value
		"""
		# Get the value from the config
		value = ConfigurationTool.getConfigRow(key)

		# Check if there is an value
		if value is None:
			return None

		# Check if global value exists
		if ConfigType.GLOBAL in value:
			return value[ConfigType.GLOBAL]

		return None

	def setValue(self, key, value):
		""" Set a config value
		"""
		self.config[key] = value
		ConfigurationTool.setConfig(key, value, ConfigType.GLOBAL)

	def _saveConfig(self):
		""" Save the config to file
		"""
		# Save the config
		ConfigurationTool.saveConfig(ConfigType.GLOBAL)

		# Everything is done
		print("All done! You can now use pete init or deploy in your projects", "yellow")
		return True



import os
import sys

from PyInquirer import prompt, Separator
from termcolor import cprint as print

from .iaction import IAction
from ..tools.configuration import ConfigurationTool, ConfigType, ConfigKey
from ..template.template import CLOUDFORMATION_TEMPLATE


class CreateProjectAction(IAction):
	""" The create project action
	"""

	def start(self):
		""" Start the create project action
		"""
		# Just a friendly message
		print("Lets setup this project, I will now ask you some questions. Feel free to answer them.\n", "yellow")

		# Get the global config
		ConfigurationTool.readConfig(getGlobal=True, getProject=True, getLocal=False)

		# Fill temporary config
		self.config = {}
		self.config[ConfigKey.STACK_NAME] = self._getConfigValue(ConfigKey.STACK_NAME)
		self.config[ConfigKey.DEV_SUFFIX] = self._getConfigValue(ConfigKey.DEV_SUFFIX)
		self.config[ConfigKey.DEV_PROFILE] = self._getConfigValue(ConfigKey.DEV_PROFILE)
		self.config[ConfigKey.PROD_PROFILE] = self._getConfigValue(ConfigKey.PROD_PROFILE)
		self.config[ConfigKey.DEV_BUCKET] = self._getConfigValue(ConfigKey.DEV_BUCKET)
		self.config[ConfigKey.PROD_BUCKET] = self._getConfigValue(ConfigKey.PROD_BUCKET)
		self.config[ConfigKey.DEV_REGION] = self._getConfigValue(ConfigKey.DEV_REGION)
		self.config[ConfigKey.PROD_REGION] = self._getConfigValue(ConfigKey.PROD_REGION)

		# Check STACK_NAME
		if self.config[ConfigKey.STACK_NAME] is None:
			self.config[ConfigKey.STACK_NAME] = os.path.split(os.getcwd())[1]

		# Check DEV_SUFFIX
		if self.config[ConfigKey.DEV_SUFFIX] is None:
			self.config[ConfigKey.DEV_SUFFIX] = True

		# Keep showing main config
		while True:
			answer = self._showMainConfig()

			if answer == 1:
				# Ask for name
				self.config[ConfigKey.STACK_NAME] = self._askName(default=self.config[ConfigKey.STACK_NAME])

			elif answer == 2:
				# Ask if we need to use a suffix
				print("Do you want to use a suffix for the development stack?\nThis can help you distinguish the difference between the stacks if you use the same account for development and production", "yellow")
				self.config[ConfigKey.DEV_SUFFIX] = self._askSuffix(default=self.config[ConfigKey.DEV_SUFFIX])

			elif answer == 4:
				# Ask if we want to override the deployment credentials
				devProfileOverride = self._askOverride(default=True if self.config[ConfigKey.DEV_PROFILE][0] != "*" else False)
				if devProfileOverride is True:
					# Ask which profile you want to use for deployment
					self.config[ConfigKey.DEV_PROFILE] = self._askAWSProfile(default=self.config[ConfigKey.DEV_PROFILE])
				else:
					# Return to global value
					self.config[ConfigKey.DEV_PROFILE] = self._getConfigValue(ConfigKey.DEV_PROFILE, getGlobal=True)

			elif answer == 5:
				# Ask if we want to override the development bucket
				devBucketOverride = self._askOverride(default=True if self.config[ConfigKey.DEV_BUCKET][0] != "*" else False)
				if devBucketOverride is True:
					# Ask the name of the Deployment bucket
					self.config[ConfigKey.DEV_BUCKET] = self._askS3Bucket(
						profile=self.config[ConfigKey.DEV_PROFILE],
						default=self.config[ConfigKey.DEV_BUCKET]
					)
				else:
					# Return to global value
					self.config[ConfigKey.DEV_BUCKET] = self._getConfigValue(ConfigKey.DEV_BUCKET, getGlobal=True)

			elif answer == 6:
				# Ask if you want to override the region
				self.config[ConfigKey.DEV_REGION] = self._askAWSRegion(
					profile=self.config[ConfigKey.DEV_PROFILE],
					default=self.config[ConfigKey.DEV_REGION]
				)

			elif answer == 8:
				# Ask if we want to override the production credentials
				prodProfileOverride = self._askOverride(default=True if self.config[ConfigKey.PROD_PROFILE][0] != "*" else False)
				if prodProfileOverride is True:
					# Ask which profile you want to use for production
					self.config[ConfigKey.PROD_PROFILE] = self._askAWSProfile(default=self.config[ConfigKey.PROD_PROFILE])
				else:
					# Return to global value
					self.config[ConfigKey.PROD_PROFILE] = self._getConfigValue(ConfigKey.PROD_PROFILE, getGlobal=True)

			elif answer == 9:
				# Ask if we want to override the product bucket
				prodBucketOverride = self._askOverride(default=True if self.config[ConfigKey.PROD_BUCKET][0] != "*" else False)
				if prodBucketOverride is True:
					# Ask the name of the Deployment bucket
					self.config[ConfigKey.PROD_BUCKET] = self._askS3Bucket(
						profile=self.config[ConfigKey.PROD_PROFILE],
						default=self.config[ConfigKey.PROD_BUCKET]
					)
				else:
					# Return to global value
					self.config[ConfigKey.PROD_BUCKET] = self._getConfigValue(ConfigKey.PROD_BUCKET, getGlobal=True)

			elif answer == 10:
				# Ask if you want to override the region
				self.config[ConfigKey.PROD_REGION] = self._askAWSRegion(
					profile=self.config[ConfigKey.PROD_PROFILE],
					default=self.config[ConfigKey.PROD_REGION]
				)

			elif answer == 11:
				# Save the config
				saved = self._saveConfig()
				if saved is True:
					sys.exit()

	def _showMainConfig(self):
		""" Show the main config options
		"""
		# Make the options for the list
		choices = []
		choices.append(Separator("~ Project ~"))
		choices.append("  Name: %s" % self.config[ConfigKey.STACK_NAME])
		choices.append("  Use DEV suffix: %s" % self.config[ConfigKey.DEV_SUFFIX])
		choices.append(Separator("~ Development ~"))
		choices.append("  AWS profile: %s" % self.config[ConfigKey.DEV_PROFILE])
		choices.append("  AWS bucket: %s" % self.config[ConfigKey.DEV_BUCKET])
		choices.append("  AWS region: %s" % self.config[ConfigKey.DEV_REGION])
		choices.append(Separator("~ Production ~"))
		choices.append("  AWS profile: %s" % self.config[ConfigKey.PROD_PROFILE])
		choices.append("  AWS bucket: %s" % self.config[ConfigKey.PROD_BUCKET])
		choices.append("  AWS region: %s" % self.config[ConfigKey.PROD_REGION])
		choices.append("Done")

		# Show the list
		answer = prompt({
			"type": "list",
			"name": "option",
			"message": "Choose an option to configure",
			"choices": choices
		})

		# Check if answer is empty
		if answer == {}:
			sys.exit()

		# Get the index of the option
		index = choices.index(answer['option'])

		return index

	def _getConfigValue(self, key, getGlobal=False):
		""" Get the config value
		"""
		# Get the value from the config
		value = ConfigurationTool.getConfigRow(key)

		# Check if there is an value
		if value is None:
			return None

		# Check if the project value exists
		if ConfigType.PROJECT in value and getGlobal is False:
			return value[ConfigType.PROJECT]

		# Check if global value exists
		elif ConfigType.GLOBAL in value:
			return "*" + value[ConfigType.GLOBAL]

		return None

	def _saveConfig(self):
		""" Save the config
		"""
		# Create the config
		config = {}
		config[ConfigKey.STACK_NAME] = self.config[ConfigKey.STACK_NAME]
		config[ConfigKey.DEV_SUFFIX] = self.config[ConfigKey.DEV_SUFFIX]

		# Check to see if we overridden anything
		if self.config[ConfigKey.DEV_PROFILE][0] != "*":
			config[ConfigKey.DEV_PROFILE] = self.config[ConfigKey.DEV_PROFILE]
		else:
			config[ConfigKey.DEV_PROFILE] = None
		if self.config[ConfigKey.PROD_PROFILE][0] != "*":
			config[ConfigKey.PROD_PROFILE] = self.config[ConfigKey.PROD_PROFILE]
		else:
			config[ConfigKey.PROD_PROFILE] = None
		if self.config[ConfigKey.DEV_BUCKET][0] != "*":
			config[ConfigKey.DEV_BUCKET] = self.config[ConfigKey.DEV_BUCKET]
		else:
			config[ConfigKey.DEV_BUCKET] = None
		if self.config[ConfigKey.PROD_BUCKET][0] != "*":
			config[ConfigKey.PROD_BUCKET] = self.config[ConfigKey.PROD_BUCKET]
		else:
			config[ConfigKey.PROD_BUCKET] = None
		if self.config[ConfigKey.DEV_REGION] not in [None, "", "<empty>"]:
			config[ConfigKey.DEV_REGION] = self.config[ConfigKey.DEV_REGION]
		else:
			config[ConfigKey.DEV_REGION] = None
		if self.config[ConfigKey.DEV_REGION] not in [None, "", "<empty>"]:
			config[ConfigKey.PROD_REGION] = self.config[ConfigKey.PROD_REGION]
		else:
			config[ConfigKey.PROD_REGION] = None

		# Check if we have a path
		try:
			ConfigurationTool._getProjectPath()
		except Exception:
			os.makedirs(".pete")

		# Set the value
		for key in config:
			ConfigurationTool.setConfig(key, config[key], ConfigType.PROJECT)

		# Save the config
		ConfigurationTool.saveConfig(ConfigType.PROJECT)

		# Check if there is on template.yaml
		if os.path.exists("template.yaml") is False:
			# Ask if we should create a CloudFormation template
			if self._askConfirm("Do you want to create a CloudFormation template?") is True:
				# Create an empty CloudFormation template
				self._createCloudFormationTemplateFile()

		# Everything is done
		print("All done! You can now use pete deploy in this project. Want to change some details? Just run init again.", "yellow")
		return True

	def _askSuffix(self, default=True):
		""" Ask if you want to the _DEV suffix
		"""
		# Ask the question
		answer = prompt({
			"type": "confirm",
			"message": "Do you want to use the development suffix?",
			"name": "suffix",
			"default": default
		})

		# Check if there is an answer
		if answer == {}:
			sys.exit()

		return answer['suffix']

	def _createCloudFormationTemplateFile(self):
		""" Create a template.yaml CloudFormation file
		"""
		# Write the content of CLOUDFORMATION_TEMPLATE to the file
		f = open("template.yaml", "w")
		f.write(CLOUDFORMATION_TEMPLATE)
		f.close()



from .actions.configure import ConfigureAction
from .actions.create_project import CreateProjectAction
from .actions.deploymentaction import DeploymentAction
from .actions.localoverride import SetupLocalOverrideAction


class ActionManager(object):
	@classmethod
	def configure(cls):
		""" Configure TPD Pete
		"""
		return ConfigureAction().start()

	@classmethod
	def createProject(cls, local=False):
		""" Create a new project
		"""
		# Check if this is the local configuation override action
		if local is True:
			return SetupLocalOverrideAction().start()

		return CreateProjectAction().start()

	@classmethod
	def deploy(cls, environType):
		""" Deploy the current project
		"""
		return DeploymentAction().start(environType)



import os
import sys

from PyInquirer import prompt, Separator
from termcolor import cprint as print

from .iaction import IAction
from ..tools.configuration import ConfigurationTool, ConfigType, ConfigKey
from ..validator import Validator


class SetupLocalOverrideAction(IAction):
	""" The setup action
	"""

	def start(self):
		""" Start the setup action
		"""
		# Check if the project has been setup
		if Validator.hasPeteProjectSetup() is False:
			print("You will need to setup the project first, by executing pete init", "orange")
			return

		# Just a friendly message
		print("Lets setup the overrides for the project, I will now ask you some questions. Feel free to answer them.\n", "yellow")

		# Get the global config
		ConfigurationTool.readConfig(getGlobal=True, getProject=True, getLocal=True)

		# Fill temporary config
		self.config = {}
		self.config[ConfigKey.DEV_PROFILE] = self._getConfigValue(ConfigKey.DEV_PROFILE)
		self.config[ConfigKey.DEV_BUCKET] = self._getConfigValue(ConfigKey.DEV_BUCKET)
		self.config[ConfigKey.DEV_REGION] = self._getConfigValue(ConfigKey.DEV_REGION)

		# Keep showing main config
		while True:
			answer = self._showMainConfig()

			if answer == 1:
				# Ask if we want to override the deployment credentials
				devProfileOverride = self._askOverride(default=True if self.config[ConfigKey.DEV_PROFILE][0] != "*" else False)
				if devProfileOverride is True:
					# Ask which profile you want to use for deployment
					self.config[ConfigKey.DEV_PROFILE] = self._askAWSProfile(default=self.config[ConfigKey.DEV_PROFILE])
				else:
					# Return to global value
					self.config[ConfigKey.DEV_PROFILE] = self._getConfigValue(ConfigKey.DEV_PROFILE, getGlobal=True, getProject=True)

			elif answer == 2:
				# Ask if we want to override the development bucket
				devBucketOverride = self._askOverride(default=True if self.config[ConfigKey.DEV_BUCKET][0] != "*" else False)
				if devBucketOverride is True:
					# Ask the name of the Deployment bucket
					self.config[ConfigKey.DEV_BUCKET] = self._askS3Bucket(
						profile=self.config[ConfigKey.DEV_PROFILE],
						default=self.config[ConfigKey.DEV_BUCKET]
					)
				else:
					# Return to global value
					self.config[ConfigKey.DEV_BUCKET] = self._getConfigValue(ConfigKey.DEV_BUCKET, getGlobal=True, getProject=True)

			elif answer == 3:
				# Ask if you want to override the region
				self.config[ConfigKey.DEV_REGION] = self._askAWSRegion(
					profile=self.config[ConfigKey.DEV_PROFILE],
					default=self.config[ConfigKey.DEV_REGION]
				)

			elif answer == 4:
				# Save the config
				saved = self._saveConfig()
				if saved is True:
					sys.exit()

	def _showMainConfig(self):
		""" Show the main config options
		"""
		# Make the options for the list
		choices = []
		choices.append(Separator("~ Development ~"))
		choices.append("  AWS profile: %s" % self.config[ConfigKey.DEV_PROFILE])
		choices.append("  AWS bucket: %s" % self.config[ConfigKey.DEV_BUCKET])
		choices.append("  AWS region: %s" % self.config[ConfigKey.DEV_REGION])
		choices.append("Done")

		# Show the list
		answer = prompt({
			"type": "list",
			"name": "option",
			"message": "Choose an option to configure",
			"choices": choices
		})

		# Check if answer is empty
		if answer == {}:
			sys.exit()

		# Get the index of the option
		index = choices.index(answer['option'])

		return index

	def _saveConfig(self):
		""" Save the config
		"""
		# Create the config
		config = {}

		# Check to see if we overridden anything
		if self.config[ConfigKey.DEV_PROFILE][0] != "*":
			config[ConfigKey.DEV_PROFILE] = self.config[ConfigKey.DEV_PROFILE]
		else:
			config[ConfigKey.DEV_PROFILE] = None
		if self.config[ConfigKey.DEV_BUCKET][0] != "*":
			config[ConfigKey.DEV_BUCKET] = self.config[ConfigKey.DEV_BUCKET]
		else:
			config[ConfigKey.DEV_BUCKET] = None
		if self.config[ConfigKey.DEV_REGION] not in [None, "", "<empty>"]:
			config[ConfigKey.DEV_REGION] = self.config[ConfigKey.DEV_REGION]
		else:
			config[ConfigKey.DEV_REGION] = None

		# Set the value
		for key in config:
			ConfigurationTool.setConfig(key, config[key], ConfigType.LOCAL)

		# Save the config
		ConfigurationTool.saveConfig(ConfigType.LOCAL)

		# Check if there is on template.yaml
		if os.path.exists("template.yaml") is False:
			self._createCloudFormationTemplateFile()

		# Everything is done
		print("All done! You can now use pete deploy in this project. Want to change some details? Just run init again.", "yellow")
		return True

	def _getConfigValue(self, key, getGlobal=False, getProject=False):
		""" Get the config value
		"""
		# Get the value from the config
		value = ConfigurationTool.getConfigRow(key)

		# Check if there is an value
		if value is None:
			return None

		# Check if the local value exists
		if ConfigType.LOCAL in value and getProject is False:
			return value[ConfigType.LOCAL]

		# Check if the project value exists
		if ConfigType.PROJECT in value and getGlobal is False:
			return value[ConfigType.PROJECT]

		# Check if global value exists
		elif ConfigType.GLOBAL in value:
			return "*" + value[ConfigType.GLOBAL]

		return None



import os
import subprocess

from termcolor import cprint as print

from ...tools.hooksmanager import hooksManager
from ...tools.configuration import ConfigurationTool
from .ideploymentaction import IDeploymentAction
from ...enums import EnvironmentType, HookTypes


class ZappaDeployment(IDeploymentAction):
	""" Deployment through Zappa
	"""
	@classmethod
	def shouldExecute(cls):
		""" Check if the Action should be deployed
		"""
		# Check if there is an zappa folder
		return os.path.exists("zappa_settings.json")

	def start(self, environType, **kwargs):
		""" Start the deployment
		"""
		# Get the configs
		ConfigurationTool.readConfig()

		# Check if there is an template
		if os.path.exists("zappa_settings.json") is False:
			raise Exception("Cant find Zappa template: 'zappa_settings.json'")

		# Remember the environment type for later
		self.environment = environType

		# Check if it is the production environment
		if self.environment == EnvironmentType.PRODUCTION:
			# Check of we een stagename override hebben
			if ConfigurationTool.getExtraConfig("prod-zappa-stage") is not None:
				stageName = ConfigurationTool.getExtraConfig("prod-zappa-stage")
			else:
				stageName = "production"
		else:
			# Check of we een stagename override hebben
			if ConfigurationTool.getExtraConfig("dev-zappa-stage") is not None:
				stageName = ConfigurationTool.getExtraConfig("dev-zappa-stage")
			else:
				stageName = "dev"

		# Check of het een array is
		if isinstance(stageName, list) is True:
			for sName in stageName:
				# Deploy the stage
				self._deployStage(sName)

		else:
			# Deploy the stage
			self._deployStage(stageName)

	def _deployStage(self, stageName):
		""" Run the zappa command
		"""
		# Execute the PRE_COMPILE hooks
		hooksManager.executeHooks(self.environment, HookTypes.PRE_COMPILE)

		# Create a temporary directory
		print("Creating environment")
		self._createTempDir()

		# Execute the POST_COMPILE hooks
		hooksManager.executeHooks(self.environment, HookTypes.POST_COMPILE)

		# Execute the PRE_INSTALL hooks
		hooksManager.executeHooks(self.environment, HookTypes.PRE_INSTALL)

		# Check the dependencies
		print("Installing dependencies")
		self._checkDependencies()

		# Execute the POST_INSTALL hooks
		hooksManager.executeHooks(self.environment, HookTypes.POST_INSTALL)

		# Execute the PRE_DEPLOYMENT hooks
		hooksManager.executeHooks(self.environment, HookTypes.PRE_DEPLOYMENT)

		# Zapping it
		print("Zappa-ing: %s" % stageName)
		# Find how to run zappa
		zappaCommand = self._findZappa()

		# Check if we could find a zappa
		if zappaCommand is None:
			raise Exception("Could not find Zappa. Make sure it is installed.")

		# Run the deployment command
		try:
			subprocess.check_output("cd %s && %s deploy %s" % (self.location, zappaCommand, stageName), stderr=subprocess.STDOUT, shell=True)
		except subprocess.CalledProcessError as e:
			# Check if it was already deployed
			if b"did you mean to call update" in e.output:
				# Run the update command
				try:
					subprocess.check_output("cd %s && %s update %s" % (self.location, zappaCommand, stageName), stderr=subprocess.STDOUT, shell=True)
				except subprocess.CalledProcessError as e:
					print(e.output, "red")
					return False
			else:
				print("An error occured", "red")
				return False
		finally:
			# Execute the POST_DEPLOYMENT hooks
			hooksManager.executeHooks(self.environment, HookTypes.POST_DEPLOYMENT)

	def _findZappa(self):
		""" Search for Zappa
		"""
		# Check if zappa is available as a root level
		try:
			subprocess.check_output("zappa", shell=True)
		except subprocess.CalledProcessError:
			pass
		else:
			return "zappa"

		# Check if poetry is available
		if os.path.exists("poetry.lock") is True:
			# Try zappa in poetry
			try:
				subprocess.check_output("poetry run zappa", shell=True)
			except subprocess.CalledProcessError:
				pass
			else:
				return "poetry run zappa"

		return None



import os
import subprocess
import sys
import tempfile
from enum import Enum

from ..iaction import IAction


class EnvironmentEnum(Enum):
	DEVELOPMENT = "development"
	PRODUCTION = "production"

	def __str__(self):
		return self.value


class IDeploymentAction(IAction):
	""" Interface for Deployment Action
	"""

	@classmethod
	def shouldExecute(cls):
		""" Check if the Action should be deployed
		"""
		raise NotImplementedError

	def __init__(self):
		""" Init the action
		"""
		# Initialize the parent
		super().__init__()

		# Set the default environment to development
		self.environment = EnvironmentEnum.DEVELOPMENT

	def _createTempDir(self):
		""" Create a temporary directory for deployment

			Returns path
		"""
		# Create the directory
		self.location = tempfile.mkdtemp()

		# Copy all the current files to it
		subprocess.check_call("cp -RL %s %s" % (".", self.location), shell=True)

		# Verwijder de pycache/pyd
		subprocess.check_call("find . | grep -E \"(__pycache__|\.pyc|\.pyo$)\" | xargs rm -rf", shell=True)

		return self.location

	def _checkDependencies(self):
		""" Check which dependencies we should install
		"""
		# Check NodeJS 'package.json'
		if os.path.exists("package.json") is True:
			subprocess.check_call("npm install --production --ignore-scripts --no-audit --prefix %s" % self.location, shell=True)

		# Check Python 'requirements.txt'
		if os.path.exists("requirements.txt") is True:
			subprocess.check_call("pip install -r requirements.txt -t %s" % self.location, shell=True)

		# Check Python 'poetry.lock'
		if os.path.exists("poetry.lock") is True:
			subprocess.check_call("poetry install", shell=True)
			os.environ["VIRTUAL_ENV"] = os.popen("poetry env info -p").read().strip()

		# Check Python virtualenv
		if os.getenv("VIRTUAL_ENV") is not None:
			packagesPath = os.path.join(os.getenv("VIRTUAL_ENV"), "lib", "python%i.%i" % (sys.version_info[0], sys.version_info[1]), "site-packages")
			subprocess.check_call("cp -R %s/. %s/" % (packagesPath, self.location), shell=True)



import argparse
import platform

from . import VERSION
from .actionmanager import ActionManager
from .validator import Validator
from .enums import EnvironmentType


class Pete(object):
	""" TPD Pete
	"""

	def __init__(self):
		# Check if Linux
		if platform.system() != "Linux":
			raise Exception("This can only be used on Linux machines")

	def start(self):
		""" Start TPD Pete
		"""
		# Create an argument parser
		parser = argparse.ArgumentParser(prog="pete", description="TPD Pete is an AWS deployment tool for AWS Cloudformation")
		parser.add_argument("mode", choices=["configure", "init", "deploy", "version"], help="Select a mode")
		parser.add_argument("--production", help="Deploy a project to your production AWS profile", action="store_true")
		parser.add_argument("--local", help="Override project setup with local development overrides", action="store_true")

		# Parse the arguments
		args = parser.parse_args()

		# Check if we want to see the version
		if args.mode == "version":
			print("TPD Deployment - Project Pete - v%s" % VERSION)
			return

		# Check if we used the configure mode
		if args.mode == "configure":
			return ActionManager.configure()

		# Check if we have a global configured Pete
		if Validator.hasPeteSetup() is False:
			raise Exception("Error: You need to setup Pete first! Use the configure mode, to setup Pete.")

		# Check if we used the init mode
		if args.mode == "init":
			return ActionManager.createProject(local=args.local)

		# Check if we have a project configured Pete
		if Validator.hasPeteProjectSetup() is False:
			raise Exception("Error: You need to init the project first! Use the init mode, to create a project of Pete.")

		# Check if we used the deploy mode
		if args.mode == "deploy":
			# Check what environment type we are going to use
			environType = EnvironmentType.DEVELOPMENT
			if args.production is True:
				environType = EnvironmentType.PRODUCTION

			return ActionManager.deploy(environType)



from tpd_pete.pete import Pete
import tpd_pete


def main():
	""" Start TPD Pete
	"""
	p = Pete()
	p.start()


if __name__ == "__main__":
	print("TPD Deployment - Project Pete - v%s" % tpd_pete.VERSION)
	main()
